"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const twing_1 = require("twing");
const fs_1 = require("fs");
const slash = require('slash');
class Visitor {
    constructor(loader, from, getTemplateHash) {
        this._loader = loader;
        this._from = new twing_1.TwingSource('', from, from);
        this._getTemplateHash = getTemplateHash;
        this._foundTemplateNames = [];
    }
    get foundTemplateNames() {
        return this._foundTemplateNames;
    }
    visit(node) {
        let processExpressionNode = (node) => {
            let pushValue = (value) => {
                if (this._loader.exists(value, this._from)) {
                    value = this._loader.resolve(value, this._from);
                    if (fs_1.existsSync(value)) {
                        if (!this._foundTemplateNames.includes(value)) {
                            this._foundTemplateNames.push(value);
                        }
                        value = this._getTemplateHash(slash(value));
                    }
                }
                return value;
            };
            if (node.getType() === twing_1.TwingNodeType.EXPRESSION_ARRAY) {
                for (let [index, constantNode] of node.getNodes()) {
                    if (index % 2) {
                        processExpressionNode(constantNode);
                    }
                }
            }
            if (node.getType() === twing_1.TwingNodeType.EXPRESSION_CONDITIONAL) {
                let expr2 = node.getNode('expr2');
                let expr3 = node.getNode('expr3');
                processExpressionNode(expr2);
                processExpressionNode(expr3);
            }
            if (node.getType() === twing_1.TwingNodeType.EXPRESSION_CONSTANT) {
                node.setAttribute('value', pushValue(node.getAttribute('value')));
            }
        };
        // include function
        if ((node.getType() === twing_1.TwingNodeType.EXPRESSION_FUNCTION) && (node.getAttribute('name') === 'include')) {
            processExpressionNode(node.getNode('arguments').getNode(0));
        }
        // import and include tags
        if ((node.getType() === twing_1.TwingNodeType.IMPORT) || (node.getType() === twing_1.TwingNodeType.INCLUDE)) {
            if (node.hasNode('expr')) {
                processExpressionNode(node.getNode('expr'));
            }
        }
        // extends and embed tags
        if ((node.getType() === twing_1.TwingNodeType.MODULE)) {
            if (node.hasNode('parent')) {
                processExpressionNode(node.getNode('parent'));
            }
            for (let embeddedTemplate of node.getAttribute('embedded_templates')) {
                this.visit(embeddedTemplate);
            }
        }
        for (let [key, subNode] of node.getNodes()) {
            this.visit(subNode);
        }
    }
    ;
}
exports.Visitor = Visitor;
